<xsl:stylesheet 
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="text" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:param name="OUTPUT_DIR"                           required="yes"/>
    <xsl:param name="NETWORK_FILE" select="network"        required="no"/>
    <xsl:param name="NETWORK"      select="$NETWORK_FILE"  required="no"/>
    <xsl:param name="VERSION"                              required="yes"/>
    <xsl:param name="ANALYSIS"                             required="no"/>
      
    <xsl:variable name="OUTPUT_DIR-LATEX-FIX">
      <xsl:value-of select="replace(OUTPUT_DIR,'_','-')"/>
    </xsl:variable>
  
    <xsl:variable name="NETWORK_FILE-LATEX-FIX">
      <xsl:value-of select="replace($NETWORK_FILE,'_','-')"/>
    </xsl:variable>
    
    <xsl:variable name="NETWORK-LATEX-FIX">
      <xsl:value-of select="replace($NETWORK,'_','-')"/>
    </xsl:variable>
    
   <xsl:variable name="ANALYSIS-LATEX-FIX">
      <xsl:value-of select="replace($ANALYSIS,'_','-')"/>
    </xsl:variable>
  
    <xsl:template match="/">

      <xsl:variable name="full-id">
	<xsl:value-of select="network/@id"/>
	<xsl:if test="network/@kind">
	  <text>-</text>
	  <xsl:value-of select="network/@kind"/>
	</xsl:if>
      </xsl:variable>


      <text xml:space="preserve">      
\documentclass{Latex/llncs}

\usepackage[a2paper]{geometry}
\usepackage[colorlinks]{hyperref}

\usepackage{longtable}
\usepackage{capt-of}

\usepackage{amsmath}
\usepackage{amssymb}
\usepackage{natbib}
\usepackage{stmaryrd}
\usepackage{mathrsfs}
\usepackage{bussproofs}
\usepackage{graphicx}
\usepackage{color}
\usepackage{multirow}
\usepackage[T1]{fontenc}
\usepackage{Latex/inference}
\usepackage{xspace}
\usepackage{listings}
\usepackage[version=3]{mhchem}
\usepackage{pdftexcmds} % for switch-macros
\usepackage{Latex/macros}
      </text>
      
      <xsl:if test="not(//*[@latex-look])">
	<text>
	  \usepackage{Latex/macros-species}
	  \usepackage{Latex/switch-macros}
	</text>
      </xsl:if>

      <xsl:if test="//*[@latex-look]">
	<text>
	\usepackage{</text>
	<xsl:value-of select="$OUTPUT_DIR"/><text>/latex-macros} 
	</text>
      </xsl:if>

	
  <text>\usepackage[dvipsnames]{xcolor}
  \usepackage{tikz}
  \usetikzlibrary{math} %needed tikz library
  \definecolor{lightblue}{HTML}{92dcec}

\usepackage{Latex/tikz-macros}

\usetikzlibrary{decorations.pathmorphing}


\EnableBpAbbreviations

\renewcommand\TIKZID[1]{</text>
      <xsl:value-of select="$full-id"/>
      <xsl:text xml:space="preserve">}

\begin{document}

\title{BioComputing's Network-Graph Tool\\
      Version </xsl:text>
      <xsl:value-of select="$VERSION"/>
      <text>}
\author{Joachim Niehren}
\institute{Inria \&amp; BioComputing, Lille, France}
\maketitle

\section{Reaction Network </text> 
<xsl:value-of select="$NETWORK_FILE-LATEX-FIX"/>
<text>}

\paragraph{Reaction Network.}

See file: \url{</text> <xsl:copy-of select="$NETWORK_FILE-LATEX-FIX"/> <text xml:space="preserve">}
See Figure \ref{fig:</text><xsl:value-of select="$NETWORK-LATEX-FIX"/><text>}.

\paragraph{Analysis}

 \url{</text> <xsl:copy-of select="$ANALYSIS-LATEX-FIX"/> <text xml:space="preserve">}



\begin{center}
\input{</text>
<xsl:value-of select="$OUTPUT_DIR"/>
<text>/graph.tex}

\captionof{figure}{\label{fig:</text><xsl:value-of select="$NETWORK_FILE"/><text>}
The </text><xsl:value-of select="$NETWORK_FILE-LATEX-FIX"/><text>.}
\addtocounter{table}{-1}
\end{center}

\begin{figure}
\centering
\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/molecules.tex}
\caption{Molecules of </text><xsl:value-of select="$NETWORK-LATEX-FIX"/><text>.}
\end{figure}

\begin{center}
\input{</text><xsl:value-of select="$OUTPUT_DIR"/><text>/reactions.tex}
\captionof{figure}{Reactions of </text><xsl:value-of select="$NETWORK_FILE-LATEX-FIX"/> <text>}.
\addtocounter{table}{-1}
\end{center}



\subsection{What Else}

\input{</text>
<xsl:value-of select="$OUTPUT_DIR"/>
<text>/comments.tex}

\end{document}</text>
    </xsl:template>


</xsl:stylesheet>
