<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    


    <xsl:template match="difference-constraints">
      <xsl:variable name="id-constraints">
	<xsl:apply-templates select="." mode="add-id"/>
      </xsl:variable>
 
      <xsl:message>
	id-constraints
	<xsl:copy-of select="$id-constraints"/>
      </xsl:message>
 
      <xsl:apply-templates select="$id-constraints" mode="flatten"/>
    </xsl:template>

    <!-- add-id -->
    <xsl:template match="difference-constraints" mode="add-id">
      <difference-constraints>
	<xsl:apply-templates select="child::*" mode="add-id"/>
      </difference-constraints>
    </xsl:template>

    <xsl:template match="speed|conc|var|input|output|knockout|diff-equal|comment|member" mode="add-id">
	<xsl:copy-of select="."/>
    </xsl:template>

   <xsl:template match="*" mode="add-id">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:attribute name="var-id">
	  <xsl:value-of select="generate-id(.)"/>
	</xsl:attribute>
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="child::*" mode="add-id"/>
      </xsl:element>
    </xsl:template>

    <!-- flatten -->
    <xsl:template match="difference-constraints" mode="flatten">
      <difference-constraints>
	<xsl:apply-templates select="child::*" mode="flatten"/>
      </difference-constraints>
    </xsl:template>

     
    <xsl:template match="equal[child::*[1][self::sum]][child::*[2][self::sum]]" mode="flatten">
      <sum> 
	<xsl:apply-templates select="child::*[1]/child::*" mode="variable"/>
	<sum>
	  <xsl:apply-templates select="child::*[2]/child::*" mode="variable"/>
	</sum>
      </sum>
    </xsl:template>

   <xsl:template match="equal[child::*[1][self::sum]][child::*[2][self::input or self::output or self::conc or self::var]]" mode="flatten">
      <sum> 
	<xsl:apply-templates select="child::*[1]/child::*" mode="variable"/>
	<xsl:apply-templates select="child::*[2]" mode="variable"/>
      </sum>
    </xsl:template>


   <xsl:template match="equal[child::*[1][self::inh]][child::*[2][self::output or self::input or self::knockout or self::speed or self::conc]]" mode="flatten">
      <inh> 
	<xsl:apply-templates select="child::*[1]/child::*" mode="variable"/>
	<xsl:apply-templates select="child::*[2]" mode="variable"/>
      </inh>
    </xsl:template>

    

   <xsl:template match="equal[child::*[1][self::acc]][child::*[2][self::output or self::input or self::knockout or self::speed or self::conc]]" mode="flatten">
      <acc> 
	<xsl:apply-templates select="child::*[1]/child::*" mode="variable"/>
	<xsl:apply-templates select="child::*[2]" mode="variable"/>
      </acc>
    </xsl:template>

   <xsl:template match="equal[child::*[2][self::inh]][child::*[1][self::output or self::input or self::knockout or self::speed or self::conc]]" mode="flatten">
      <inh> 
	<xsl:apply-templates select="child::*[2]/child::*" mode="variable"/>
	<xsl:apply-templates select="child::*[1]" mode="variable"/>
      </inh>
    </xsl:template>

   <xsl:template match="equal[child::*[2][self::acc]][child::*[1][self::output or self::input or self::knockout or self::speed or self::conc]]" mode="flatten">
      <acc> 
	<xsl:apply-templates select="child::*[2]/child::*" mode="variable"/>
	<xsl:apply-templates select="child::*[1]" mode="variable"/>
      </acc>
    </xsl:template>

     
   

    <xsl:template match="*[parent::equal][self::sum or self::mult]" mode="flatten">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:copy-of select="@value"/>
	<xsl:apply-templates select="child::*" mode="variable"/>
	<xsl:apply-templates select="parent::*" mode="variable"/>
      </xsl:element>
      <xsl:apply-templates select="child::*" mode="flatten"/>
    </xsl:template>

    <xsl:template match="acc | inh | binary-minus | mult" mode="flatten">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:copy-of select="@value"/>
	<xsl:apply-templates select="child::*" mode="variable"/>
	<xsl:apply-templates select="."        mode="variable"/>
      </xsl:element>
      <xsl:apply-templates select="child::*" mode="flatten"/>
    </xsl:template>
			 
    <xsl:template match="equal[not(sum) and not(mult) and not(inh) and not(acc)]" mode="flatten">
      <equal>
	<xsl:copy-of select="child::*"/>
      </equal>
    </xsl:template>
    
  
    <xsl:template match="*[parent::difference-constraints]
			 [self::sum or self::mult]" mode="flatten">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="child::*" mode="variable"/>
      </xsl:element>
      <xsl:apply-templates select="child::*" mode="flatten"/>
    </xsl:template>

    <xsl:template match="comment|member" mode="flatten">
      <xsl:copy-of select="."/>
    </xsl:template>
    
    <xsl:template match="var|conc|speed|input|output|knockout|diff-equal" mode="variable">
      <xsl:copy-of select="."/>
    </xsl:template>

    <xsl:template match="*" mode="variable">
      <var>
	<xsl:attribute name="id">
	  <xsl:copy-of select="./@var-id"/>
	</xsl:attribute>
      </var>
    </xsl:template>

    
    
</xsl:stylesheet>
