<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:variable name="EMPTY">
      <text>Targets/empty.xml</text>    
    </xsl:variable>

    <xsl:param name="TARGETS" select="$EMPTY" required="no"/>

    <xsl:variable name="targets">
      <xsl:message>
	 Targets <xsl:value-of select="$TARGETS"/>
      </xsl:message>
      <xsl:copy-of select="document(concat('../',$TARGETS))"/>
    </xsl:variable>
    
    <xsl:template match="steady-state">
      <difference-constraints>
	<comment txt="steady state constraints"/>
	<xsl:apply-templates select="equal|similar"/>
	<xsl:copy-of select="$targets//difference-constraints/child::*"/>
      </difference-constraints>
    </xsl:template>

   <xsl:template match="equal[child::*[1][self::sum]]
			[child::*[2][self::sum]]">
     <equal>
        <xsl:apply-templates select="child::*"/>
     </equal>
   </xsl:template>

   <xsl:template match="equal[child::sum][not(child::*[1][self::sum])]">
      <sum>
	<xsl:apply-templates select="child::sum/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </sum>
    </xsl:template>
   <xsl:template match="equal[mult]">
      <mult>
	<xsl:apply-templates select="child::mult/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </mult>
    </xsl:template>
    <xsl:template match="equal[apply]">
      <apply>
	<xsl:apply-templates select="apply/@kinetics"/>
	<xsl:apply-templates select="apply/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </apply>
    </xsl:template>

    <!-- otherwise -->
    <xsl:template match="*">
      <xsl:variable name="tag">
	<xsl:value-of select="name(.)"/>
      </xsl:variable>
      <xsl:element name="{$tag}"> 
	<xsl:apply-templates select="@*|child::*"/>
      </xsl:element>
    </xsl:template>
    
    <xsl:template match="@*">
	<xsl:copy-of select="."/>
    </xsl:template>

    <xsl:template match="similar[child::*[1][self::sum]]">
      <sum>
	<xsl:apply-templates select="child::*[1]/self::sum/child::*"/>
	<xsl:apply-templates select="child::*[2]"/>
      </sum>
    </xsl:template>

    <xsl:template match="similar[child::*[2][self::sum]]">
      <sum>
	<xsl:apply-templates select="child::*[2]/self::sum/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </sum>
    </xsl:template>

   <xsl:template match="similar[count(var|speed|input|output|knockout|conc)=2]">
      <equal>
	<xsl:apply-templates select="child::*"/>
      </equal>
    </xsl:template>

    <xsl:template match="similar[binary-minus][not(sum)]">
      <binary-minus>
	<xsl:apply-templates select="child::binary-minus/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </binary-minus>
    </xsl:template>

    <xsl:template match="similar[mult][not(sum)]">
      <mult>
	<xsl:apply-templates select="child::mult/child::*"/>
	<xsl:apply-templates select="child::*[1]"/>
      </mult>
    </xsl:template>

    <xsl:template match="similar[apply]">
      <apply>
	<xsl:copy-of select="apply/@kinetics"/>
	<xsl:copy-of select="apply/child::*"/>
	<xsl:copy-of select="child::*[1]"/>
      </apply>
    </xsl:template>

    <xsl:template match="similar">
      <equal>
	<xsl:apply-templates select="child::*"/>
      </equal>
    </xsl:template>

    <xsl:template match="constant">
      <diff-equal>
	  <xsl:copy-of select="@value"/>
      </diff-equal>
    </xsl:template>



     
</xsl:stylesheet>
