<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="text" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:strip-space elements="*" />
      
    <xsl:template match="//difference-constraints">
      <text xml:space="preserve">\begin{longtable}{l} 
</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">\end{longtable}
</text>
    </xsl:template>

    <xsl:template match="equal">
      <text>\qquad $</text> 
      <xsl:apply-templates select="child::*[1]"/>
      <text>  = </text> 
      <xsl:apply-templates select="child::*[2]"/>
      <text xml:space="preserve">$\\
</text>
    </xsl:template>

    <xsl:template match="sum[parent::difference-constraints]">
      <text> \qquad $ +(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)$\\
</text>
    </xsl:template>

    <xsl:template match="sum[not(parent::difference-constraints)]">
      <text> +(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)
      </text>
    </xsl:template>

    <xsl:template match="mult[parent::difference-constraints]">
      <text>  \qquad $ \cdot(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)$\\
</text>
    </xsl:template>

    <xsl:template match="mult[not(parent::difference-constraints)]">
      <text>  \cdot(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)
      </text>
    </xsl:template>


    <xsl:template match="binary-minus[parent::difference-constraints]">
      <text> \qquad $ -(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)$\\
</text>
    </xsl:template>

    <xsl:template match="binary-minus[not(parent::difference-constraints)]">
      <text>  -(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)
</text>
    </xsl:template>


    <xsl:template match="apply">
      <text> $ </text>
      <xsl:value-of select="@kinetics"/>
      <text>(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)$\\
</text>
    </xsl:template>


    <xsl:template match="diff-equal">
      <text>=_{</text> <xsl:value-of select="@value"/>
      <text>}</text>
       <xsl:if test="following-sibling::* and not(parent::equal)">
	 <text>, </text>
       </xsl:if>
    </xsl:template>

    <xsl:template match="inh[parent::difference-constraints]">
      <text>\qquad$\inh(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)$\\</text>
    </xsl:template>

    <xsl:template match="inh">
      <text>\inh(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)</text>
      <xsl:if test="following-sibling::* and not(parent::equal)">
	<text>, </text>
      </xsl:if>
    </xsl:template>

    <xsl:template match="acc[parent::difference-constraints]">
      <text>\qquad $\acc(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)$\\</text>
    </xsl:template>

    <xsl:template match="acc">
      <text>\acc(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)</text>
      <xsl:if test="following-sibling::* and not(parent::equal)">
	<text>, </text>
      </xsl:if>
    </xsl:template>

    <xsl:template match="inv">
      <text>1/(</text>
      <xsl:apply-templates select="child::*"/>
      <text xml:space="preserve">)</text>
      <xsl:if test="following-sibling::* and not(parent::equal) and not(parent::member)">
	<text>, </text>
      </xsl:if>
    </xsl:template>

    <xsl:template match="minus">
       <text>-</text>
      <xsl:apply-templates select="child::*"/>
      <xsl:if test="following-sibling::* and not(parent::equal) and not(parent::member)">
	<text>, </text>
      </xsl:if>
    </xsl:template>

    <xsl:template match="output">
      <text> \y{</text> 
      <xsl:value-of select="normalize-space(@spec)"/>
       <text xml:space="preserve">}</text>
       <xsl:if test="following-sibling::* and not(parent::equal) and not(parent::member)">
	 <text>, </text>
       </xsl:if>
    </xsl:template>

    <xsl:template match="input">
      <text> \x{</text> 
      <xsl:value-of select="normalize-space(@spec)"/>
       <text xml:space="preserve">}</text>
       <xsl:if test="following-sibling::* and not(parent::equal) and not(parent::member)">
	 <text>, </text>
       </xsl:if>
    </xsl:template>

    <xsl:template match="conc">
      <text> \z{</text> 
      <xsl:value-of select="normalize-space(@spec)"/>
       <text xml:space="preserve">}</text>
       <xsl:if test="following-sibling::* and not(parent::equal) and not(parent::member)">
	 <text>, </text>
       </xsl:if>
    </xsl:template>

    <xsl:template match="knockout">
      <text> \ko{</text> 
      <xsl:value-of select="normalize-space(@id)"/>
       <text xml:space="preserve">}</text>
       <xsl:if test="following-sibling::* and not(parent::equal) and not(parent::member)">
	 <text>, </text>
       </xsl:if>
    </xsl:template>

    <xsl:template match="var">
      <text> \w{</text> 
      <xsl:value-of select="normalize-space(@id)"/>
       <text xml:space="preserve">}</text>
       <xsl:if test="following-sibling::* and not(parent::equal) and not(parent::member)">
	<text>, </text>
       </xsl:if>
    </xsl:template>

    <xsl:template match="speed">
      <text> \v{</text> 
      <xsl:value-of select="normalize-space(@reaction)"/>
       <text xml:space="preserve">}</text>
       <xsl:if test="following-sibling::* and not(parent::equal) and not(parent::member)">
	 <text>, </text>
       </xsl:if>
    </xsl:template>
    
    <xsl:template match="comment">
      <xsl:if test="not(preceding-sibling::comment)
	and preceding-sibling::*">
	<text xml:space="preserve"> \\
</text>
      </xsl:if>
       <xsl:value-of select="@txt"/>
      <text xml:space="preserve"> \\
</text>

    </xsl:template>


		  

    <xsl:template match="member">
      <text xml:space="preserve">\qquad  $</text>
      <xsl:apply-templates select="child::*[1]"/>
      <text xml:space="preserve"> \in \{ </text>
      <xsl:for-each select="child::*[2][self::set]/child::*[not(position()=last())]">
	<xsl:apply-templates select="."/>
	<text>, </text>
      </xsl:for-each>
      <xsl:apply-templates select="child::*[2][self::set]/child::*[position()=last()]"/>
      <text xml:space="preserve"> \} $\\
</text>
    </xsl:template>


    <xsl:template match="diff">
      <text>\</text><xsl:value-of select="@value"/>
    </xsl:template>



</xsl:stylesheet>

