<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">


    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
<!--    <xsl:strip-space elements="*" /> -->

    <!-- user parameters -->

    <xsl:variable name="Delta">
      <text>6</text>   <!-- domain Delta with either 3 or 6 elements -->
    </xsl:variable>
    <xsl:variable name="debug">
      <text>no</text>  <!-- mode debug either 'yes' or 'no' -->
    </xsl:variable>

    <!-- main procedure -->

    <xsl:template match="//difference-constraints">
      <xsl:variable name="start">
	<difference-constraints change="no">
	  <xsl:copy-of select="child::*"/>
	</difference-constraints>
      </xsl:variable>
      <xsl:variable name="next">
	<xsl:apply-templates select="$start/difference-constraints" mode="simplify"/>
      </xsl:variable>
      <!-- repeat while changes -->
      <!--
      <xsl:message>
	any changes? <xsl:value-of select="$next/difference-constraints/@change"/>
      </xsl:message>
      -->
      <xsl:choose>
	<xsl:when test="$next/difference-constraints/@change='yes'">
	   <xsl:apply-templates select="$next/difference-constraints"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="$next/difference-constraints"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>
    
    <xsl:template match="difference-constraints" mode="simplify">
      <xsl:variable name="elim-equal">
	<xsl:apply-templates select="." mode="elim-equal"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  elim-equal <xsl:copy-of select="$elim-equal"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="elim-constant">
	<xsl:apply-templates select="$elim-equal" mode="elim-constant"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  elim-constant <xsl:copy-of select="$elim-constant"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="constant-inh-simplify">
	<xsl:apply-templates select="$elim-constant" mode="constant-inh-simplify"/> 
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  constant-inh-simplify <xsl:copy-of select="$constant-inh-simplify"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="elim-diff-equal">
	<xsl:apply-templates select="$constant-inh-simplify" mode="elim-diff-equal"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  elim-diff-equal <xsl:copy-of select="$elim-diff-equal"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="elim-diff-equal-last">
	<xsl:apply-templates select="$elim-diff-equal" mode="elim-diff-equal-last"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  elim-diff-equal-last <xsl:copy-of select="$elim-diff-equal-last"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="binary">
	<xsl:apply-templates select="$elim-diff-equal-last" mode="binary"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  binary 	<xsl:copy-of select="$binary"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="inh-equal-is-equal">
	<xsl:apply-templates select="$binary" mode="inh-equal-is-equal"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  inh-equal-is-equal <xsl:copy-of select="$inh-equal-is-equal"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="acc-equal-is-equal">
	<xsl:apply-templates select="$inh-equal-is-equal" mode="acc-equal-is-equal"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  acc-equal-is-equal <xsl:copy-of select="$acc-equal-is-equal"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="inh-acc-is-inh">
	<xsl:apply-templates select="$acc-equal-is-equal" mode="inh-acc-is-inh"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  inh-acc-is-inh <xsl:copy-of select="$inh-acc-is-inh"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="inv-inh-is-acc">
	<xsl:apply-templates select="$inh-acc-is-inh" mode="inv-inh-is-acc"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  inv-inh-is-acc <xsl:copy-of select="$inv-inh-is-acc"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="zero-ary-sum">
	<xsl:apply-templates select="$inv-inh-is-acc" mode="zero-ary-sum"/>
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  zero-ary-sum <xsl:copy-of select="$zero-ary-sum"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="normalize-diff-equal">
	<xsl:apply-templates select="$zero-ary-sum" mode="normalize-diff-equal"/> 
      </xsl:variable> 
      <xsl:if test="$debug='yes'">    
	<xsl:message>
	  normalize-diff-equal <xsl:copy-of select="$normalize-diff-equal"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="elim-diff-equal-one-in-sum">
	<xsl:apply-templates select="$normalize-diff-equal" mode="elim-diff-equal-one-in-sum"/> 
      </xsl:variable> 
      <xsl:if test="$debug='yes'">    
	<xsl:message>
	  elim-diff-equal-one-in-sum <xsl:copy-of select="$elim-diff-equal-one-in-sum"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="binary-minus-equal-one-minus-is-acc">
	<xsl:apply-templates select="$elim-diff-equal-one-in-sum" 
			     mode="binary-minus-equal-one-minus-is-acc"/> 
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  binary-minus-equal-one-minus-is-acc <xsl:copy-of 
	  select="$binary-minus-equal-one-minus-is-acc"/>
	</xsl:message>
      </xsl:if>
      <xsl:variable name="equal-one-in-sum">
	<xsl:apply-templates select="$binary-minus-equal-one-minus-is-acc" 
			     mode="equal-one-in-sum"/> 
      </xsl:variable>
      <xsl:if test="$debug='yes'">
	<xsl:message>
	  equal-one-in-sum <xsl:copy-of select="$equal-one-in-sum"/>
	</xsl:message>
      </xsl:if>

      <xsl:copy-of select="$equal-one-in-sum"/>
    </xsl:template>


    <!-- elim-equal -->

     <xsl:template match="difference-constraints" mode="elim-equal">
       <xsl:choose>
	 <xsl:when test="equal[speed or conc or var]">
	   <xsl:for-each select="(equal[speed or conc or var])[1]">
	     <xsl:variable name="rest-equations">
	       <difference-constraints change="yes">
		 <xsl:copy-of select="preceding-sibling::*|following-sibling::*"/>
	       </difference-constraints>
	     </xsl:variable>
	     <xsl:variable name="next-equations">
	       <xsl:apply-templates 
		   select="$rest-equations//difference-constraints" 
		   mode="query-replace">
		 <xsl:with-param name="query"   
				 select="(speed|conc|var)[1]"/>
		 <xsl:with-param name="replace" 
				 select="child::* except (speed|conc|var)[1]"/>
	       </xsl:apply-templates>
	     </xsl:variable>
	     <xsl:apply-templates select="$next-equations/difference-constraints" mode="elim-equal"/>
	   </xsl:for-each>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <!-- elim-constant -->

     <xsl:template match="difference-constraints" mode="elim-constant">
       <xsl:choose>
	 <xsl:when test="constant[speed or conc or var]">
	   <xsl:for-each select="(constant[speed or conc or var])[1]">
	     <xsl:variable name="rest-equations">
	       <difference-constraints change="yes">
		 <xsl:copy-of select="preceding-sibling::*|following-sibling::*"/>
	       </difference-constraints>
	     </xsl:variable>

	     <xsl:variable name="next-equations">
	       <xsl:apply-templates 
		   select="$rest-equations//difference-constraints" 
		   mode="query-replace">
		 <xsl:with-param name="query"    
				 select="speed|conc|var"/>
		 <xsl:with-param name="replace"> 
		   <diff-equal>
		     <xsl:copy-of select="@value"/>
		   </diff-equal>
		 </xsl:with-param>
	       </xsl:apply-templates>
	     </xsl:variable>

	     <xsl:apply-templates 
		 select="$next-equations//difference-constraints" 
		 mode="elim-constant"/>
	   </xsl:for-each>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <!-- elim-diff-equal -->

     <xsl:template match="difference-constraints" mode="elim-diff-equal">
       <xsl:choose>
	 <xsl:when test="sum[ diff-equal[following-sibling::* and @value='0']] |
			 mult[diff-equal[following-sibling::* and @value='1']] ">
	   <difference-constraints change="yes">
	     <xsl:apply-templates select="child::*" mode="elim-diff-equal"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

    <xsl:template match="sum[diff-equal[following-sibling::* and @value='0']]"  mode="elim-diff-equal">
      <xsl:variable name="return">
	<sum>
	  <xsl:copy-of select="child::*[not(position() &lt; last()
			       and self::diff-equal/@value='0')]"/>
	</sum>
      </xsl:variable>
      <!--
      <xsl:message>
	elim-diff-equal <xsl:copy-of select="."/>
        to <xsl:copy-of select="$return"/>
      </xsl:message>
      -->
      <xsl:copy-of select="$return"/>
      </xsl:template>

     <xsl:template match="mult[diff-equal[following-sibling::* and @value='1']]" mode="elim-diff-equal">
      <xsl:variable name="return">
	<mult>
	  <xsl:copy-of select="child::*[not(position() &lt; last()
			       and self::diff-equal/@value='1')]"/>
	</mult>
      </xsl:variable>
      <!--
      <xsl:message>
	elim-diff-equal <xsl:copy-of select="."/>
	to	<xsl:copy-of select="$return"/>
      </xsl:message>
      -->
      <xsl:copy-of select="$return"/>
      </xsl:template>

     <xsl:template match="* | @*"  mode="elim-diff-equal">
       <xsl:copy-of select="."/>
     </xsl:template>

     <!-- elim-diff-equal -->

     <xsl:template match="difference-constraints" mode="elim-diff-equal-last">
       <xsl:choose>
	 <xsl:when test="sum[child::*[self::diff-equal and @value='0' and position()=last()]] |
			 mult[child::*[self::diff-equal and @value='1' and position()=last()]] |
			 binary-minus[child::*[self::diff-equal and @value='0' and position()=last()]]">
	   <difference-constraints change="yes" debug="elim-diff-equal-last">
	     <xsl:apply-templates select="child::*" mode="elim-diff-equal-last"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="sum[child::*[self::diff-equal and
			  @value='0' and position()=last()]]"  
		   mode="elim-diff-equal-last">
       <xsl:variable name="output">
	 <xsl:apply-templates select="child::*[position()&lt;last()-1]" 
			      mode="elim-diff-equal-last"/>
       </xsl:variable>
       <!--
       <xsl:message>
	 elim-diff-equal-last <xsl:copy-of select="."/>
         to <xsl:copy-of select="$output"/>
       </xsl:message>
       -->
       <xsl:copy-of select="$output"/>
     </xsl:template>

     <xsl:template match="mult[child::*[self::diff-equal and
			  @value='1' and position()=last()]]"  
		   mode="elim-diff-equal-last">
       <xsl:choose>
	 <xsl:when test="count(child::*) &gt; 1">
	   <xsl:variable name="output">
	     <mult>
	       <xsl:copy-of select="child::*[position() &lt; last()-1]"/>
	       <inv> 
		 <xsl:copy-of select="child::*[position() = last()-1]"/>
	       </inv>
	     </mult>
	   </xsl:variable>
	   <!--
	   <xsl:message>
	     elim-diff-equal-last <xsl:copy-of select="."/>
	     to <xsl:copy-of select="$output"/>
	   </xsl:message>
	   -->
	   <xsl:copy-of select="$output"/>
	 </xsl:when>
       </xsl:choose>	   
     </xsl:template>

     <xsl:template match="binary-minus[child::*[self::diff-equal and
			  @value='0' and position()=last()]]"  
		   mode="elim-diff-equal-last">
       <xsl:choose>
	 <xsl:when test="count(child::*) = 3">
   	   <xsl:variable name="output">
	     <equal>
	       <xsl:copy-of select="child::*[1]"/>
	       <xsl:copy-of select="child::*[2]"/>
	     </equal>
	   </xsl:variable>
	   <!--
	   <xsl:message>
	     elim-diff-equal-last <xsl:copy-of select="."/>
	     to <xsl:copy-of select="$output"/>
	   </xsl:message>
	   -->
	   <xsl:copy-of select="$output"/>
	 </xsl:when>
       </xsl:choose>	   
     </xsl:template>

     <xsl:template match="@*|*"  mode="elim-diff-equal-last">
       <xsl:copy-of select="."/>
     </xsl:template>
     


     <xsl:template match="@*" mode="query-replace">
       <xsl:copy-of select="."/>
     </xsl:template>

     <xsl:template match="*[not(self::speed) and 
			  not(self::conc) and
			  not(self::var)]" mode="query-replace">
       <xsl:param name="query"/>
       <xsl:param name="replace"/>
       
       <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>

      <xsl:element name="{$tag}">
	<xsl:apply-templates select="child::*|@*" mode="query-replace">
	  <xsl:with-param name="query"   select="$query"/>
	  <xsl:with-param name="replace" select="$replace"/>
	</xsl:apply-templates>
      </xsl:element>

     </xsl:template>

     <xsl:template match="speed|conc|var" mode="query-replace">
       <xsl:param name="query"/>
       <xsl:param name="replace"/>

       <xsl:choose>
	 <xsl:when test="self::speed[@reaction=$query/@reaction] or
 			 self::conc[@spec=$query/@spec] or
			 self::var[@id=$query/@id]">
	 
	   <xsl:message>
	     substitute <xsl:copy-of select="$query"/> 
	      by <xsl:copy-of select="$replace"/>
	    </xsl:message>
	   <xsl:copy-of select="$replace"/>

	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="difference-constraints" mode="constant-inh-simplify">
       <xsl:choose>
	 <xsl:when test="constant[inh]">
	   <difference-constraints change="yes">
	     <xsl:for-each select="constant[inh]">
	       <constant>
		 <xsl:copy-of select="@*|inh/child::*"/>
	       </constant>
	     </xsl:for-each>
	     <xsl:copy-of select="child::*[not(self::constant[inh])]"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="difference-constraints" mode="binary">
       <xsl:choose>
	 <xsl:when test="sum[count(child::*)=2]	| mult[count(child::*)=2]">
	   <difference-constraints change="yes">
	     <xsl:apply-templates select="child::*" mode="binary"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="sum[count(child::*)=2]"  mode="binary">
       <equal>
	 <xsl:copy-of select="child::*[1]"/>
	 <xsl:copy-of select="child::*[2]"/>
       </equal>
     </xsl:template>
     <xsl:template match="mult[count(child::*)=2]"  mode="binary">
       <equal>
	 <xsl:copy-of select="child::*[1]"/>
	 <xsl:copy-of select="child::*[2]"/>
       </equal>
     </xsl:template>

     <xsl:template match="*"  mode="binary">
       <xsl:copy-of select="."/>
     </xsl:template>

    <xsl:template match="difference-constraints" mode="inh-equal-is-equal">
       <xsl:choose>
	 <xsl:when test=".//inh[diff-equal]">
	   <difference-constraints change="yes">
	     <xsl:apply-templates select="child::*" mode="inh-equal-is-equal"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="inh[diff-equal]"  mode="inh-equal-is-equal">
       <xsl:variable name="output">
	 <diff-equal>
	   <xsl:attribute name="value">
	     <xsl:copy-of select="1 div (diff-equal/@value+1)"/>
	   </xsl:attribute>
	 </diff-equal>
       </xsl:variable>
       <!--
       <xsl:message>
	 inh-equal-is-equal <xsl:copy-of select="."/>
         to <xsl:copy-of select="$output"/>
       </xsl:message>
       -->
       <xsl:copy-of select="$output"/>
     </xsl:template>
 
     <xsl:template match="*"  mode="inh-equal-is-equal">
       <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
       <xsl:element name="{$tag}">
	 <xsl:apply-templates select="@*| child::*" mode="inh-equal-is-equal"/>
       </xsl:element>
     </xsl:template>
     
     <xsl:template match="@*" mode="inh-equal-is-equal">
       <xsl:copy-of select="."/>
     </xsl:template>

    <xsl:template match="difference-constraints" mode="acc-equal-is-equal">
       <xsl:choose>
	 <xsl:when test=".//acc[diff-equal]">
	   <difference-constraints change="yes">
	     <xsl:apply-templates select="child::*" mode="acc-equal-is-equal"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="acc[diff-equal]"  mode="acc-equal-is-equal">
       <diff-equal>
	 <xsl:attribute name="value">
	   <xsl:copy-of select="min((1+ diff-equal/@value,1))"/>
	 </xsl:attribute>
       </diff-equal>
     </xsl:template>
 
     <xsl:template match="*"  mode="acc-equal-is-equal">
       <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
       <xsl:element name="{$tag}">
	 <xsl:apply-templates select="@*| child::*" mode="acc-equal-is-equal"/>
       </xsl:element>
     </xsl:template>
     
     <xsl:template match="@*" mode="acc-equal-is-equal">
       <xsl:copy-of select="."/>
     </xsl:template>

     <!-- inh-acc-is-inh --> 

    <xsl:template match="difference-constraints" mode="inh-acc-is-inh">
       <xsl:choose>
	 <xsl:when test=".//inh[acc]">
	   <difference-constraints change="yes">
	     <xsl:apply-templates select="child::*" mode="inh-acc-is-inh"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="inh[acc]"  mode="inh-acc-is-inh">
       <xsl:variable name="output">
	 <inh>
	   <xsl:apply-templates select="acc/child::*" mode="inh-acc-is-inh"/>
	 </inh>
       </xsl:variable>
       <!--
       <xsl:message>
	 inh-acc-is-inh <xsl:copy-of select="."/>
         to <xsl:copy-of select="$output"/>
       </xsl:message>
       -->
       <xsl:copy-of select="$output"/>
     </xsl:template>
 
     <xsl:template match="*"  mode="inh-acc-is-inh">
       <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
       <xsl:element name="{$tag}">
	 <xsl:apply-templates select="@*| child::*" mode="inh-acc-is-inh"/>
       </xsl:element>
     </xsl:template>
     
     <xsl:template match="@*" mode="inh-acc-is-inh">
       <xsl:copy-of select="."/>
     </xsl:template>

     <!-- inv-inh-is-acc -->

    <xsl:template match="difference-constraints" mode="inv-inh-is-acc">
       <xsl:choose>
	 <xsl:when test=".//inv[inh]">
	   <difference-constraints change="yes">
	     <xsl:apply-templates select="child::*" mode="inv-inh-is-acc"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="inv[inh]"  mode="inv-inh-is-acc">
       <xsl:variable name="output">
	 <inh>
	   <xsl:apply-templates select="inh/child::*" mode="inv-inh-is-acc"/>
	 </inh>
       </xsl:variable>
       <!--
       <xsl:message>
	 inv-inh-is-acc <xsl:copy-of select="."/>
         to <xsl:copy-of select="$output"/>
       </xsl:message>
       -->
       <xsl:copy-of select="$output"/>
     </xsl:template>
 
     <xsl:template match="*"  mode="inv-inh-is-acc">
       <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
       <xsl:element name="{$tag}">
	 <xsl:apply-templates select="@*| child::*" mode="inv-inh-is-acc"/>
       </xsl:element>
     </xsl:template>
     
     <xsl:template match="@*" mode="inv-inh-is-acc">
       <xsl:copy-of select="."/>
     </xsl:template>



   <xsl:template match="difference-constraints" 
		 mode="binary-minus-equal-one-minus-is-acc">
       <xsl:choose>
	 <xsl:when test=".//binary-minus
			 [child::*[1]/self::diff-equal[@value='1']]
			 [child::*[3]/self::minus]">
	   <difference-constraints change="yes">
	     <xsl:apply-templates select="child::*"
				  mode="binary-minus-equal-one-minus-is-acc"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="binary-minus[child::*[1]/self::diff-equal[@value='1']][child::*[3]/self::minus]" 
		   mode="binary-minus-equal-one-minus-is-acc">
       <xsl:variable name="output">
	 <equal>
	   <xsl:copy-of select="child::*[2]"/>
	   <xsl:copy-of select="child::*[3]/self::minus/child::*"/>
	 </equal>
       </xsl:variable>
       <!--
       <xsl:message>
	 binary-minus-equal-one-minus-is-acc <xsl:copy-of select="."/>
         to <xsl:copy-of select="$output"/>
       </xsl:message>
       -->
       <xsl:copy-of select="$output"/>
     </xsl:template>
 
     <xsl:template match="*"  mode="binary-minus-equal-one-minus-is-acc">
       <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
       <xsl:element name="{$tag}">
	 <xsl:apply-templates select="@*| child::*" mode="binary-minus-equal-one-minus-is-acc"/>
       </xsl:element>
     </xsl:template>
     
     <xsl:template match="@*" mode="binary-minus-equal-one-minus-is-acc">
       <xsl:copy-of select="."/>
     </xsl:template>

     <!-- normalize-diff-equal -->

     <xsl:template match="difference-constraints" mode="normalize-diff-equal">
       <xsl:choose>
	 <xsl:when test=".//diff-equal[not(@value='1' or @value='0')]
			 or .//diff-equal[@value=1 and $Delta='3']">
	   <xsl:message>
	     normalize-diff-equal applies
       </xsl:message>
	   <difference-constraints change="yes">
	     <xsl:apply-templates select="child::*" mode="normalize-diff-equal"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="diff-equal[not(@value='0' or @value='1')]" mode="normalize-diff-equal">
       <xsl:variable name="output">
	 <diff-equal value="1"/>
       </xsl:variable>
       <xsl:message>
	 normalize-diff-equal <xsl:copy-of select="."/>
	 to <xsl:copy-of select="$output"/>
       </xsl:message>
       <xsl:copy-of select="$output"/>
     </xsl:template>

     <xsl:template match="diff-equal[@value='1' and $Delta='3']" mode="normalize-diff-equal">
       <xsl:variable name="output">
	 <diff-equal value="0"/>
       </xsl:variable>
       <xsl:message>
	 Delta with 3 elements
	 normalize-diff-equal <xsl:copy-of select="."/>
	 to <xsl:copy-of select="$output"/>
       </xsl:message>
       <xsl:copy-of select="$output"/>
     </xsl:template>

 
     <xsl:template match="*" mode="normalize-diff-equal">
      <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
       <xsl:element name="{$tag}">
	 <xsl:apply-templates select="@*| child::*" mode="normalize-diff-equal"/>
       </xsl:element>
     </xsl:template>

     <xsl:template match="@*" mode="normalize-diff-equal">
       <xsl:copy-of select="."/>
     </xsl:template>

     <!-- elim-diff-equal-one-in-sum -->

     <xsl:template match="difference-constraints" mode="elim-diff-equal-one-in-sum">
       <xsl:choose>
	 <xsl:when test="sum[diff-equal[@value='1' and following-sibling::*]][acc or inh]">
	   <difference-constraints change="yes">
	     <xsl:apply-templates select="child::*" mode="elim-diff-equal-one-in-sum"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="sum[diff-equal[@value='1' and following-sibling::*]][acc or inh]" 
		   mode="elim-diff-equal-one-in-sum">
       <xsl:variable name="output">
	 <sum>
	   <xsl:copy-of select="child::*[not(self::diff-equal[@value='1' and following-sibling::*])]"/>
	 </sum>
       </xsl:variable>
       <!--
       <xsl:message>
	 elim-diff-equal-one-in-sum <xsl:copy-of select="."/>
	 to <xsl:copy-of select="$output"/>
       </xsl:message>
       -->
       <xsl:copy-of select="$output"/>
     </xsl:template>

     <xsl:template match="*" mode="elim-diff-equal-one-in-sum">
      <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
       <xsl:element name="{$tag}">
	 <xsl:apply-templates select="@*| child::*" mode="elim-diff-equal-one-in-sum"/>
       </xsl:element>
     </xsl:template>

     <xsl:template match="@*" mode="elim-diff-equal-one-in-sum">
       <xsl:copy-of select="."/>
     </xsl:template>


     <!-- zero-ary-sum -->

     <xsl:template match="difference-constraints" mode="zero-ary-sum">
       <xsl:choose>
	 <xsl:when test="sum[count(*)=1]">
	   <difference-constraints change="yes">
	     <xsl:apply-templates select="child::*" mode="zero-ary-sum"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="sum[count(*)=1]" mode="zero-ary-sum">
       <constant value="0">
	 <xsl:copy-of select="child::*[1]"/>
       </constant>
     </xsl:template>

     <xsl:template match="*" mode="zero-ary-sum">
	 <xsl:copy-of select="."/>
     </xsl:template>

    <!-- equal-one-in-sum -->

     <xsl:template match="difference-constraints" mode="equal-one-in-sum">
       <xsl:choose>
	 <xsl:when test="sum[diff-equal[@value=1]][count(diff-equal[@value=1])+1 &lt; last()]">
	   <difference-constraints change="yes">
	     <xsl:message>
	       equal-one-in-sum applies
	     </xsl:message>
	     <xsl:apply-templates select="child::*" mode="equal-one-in-sum"/>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="sum[diff-equal[@value=1]][count(diff-equal[@value=1])+1 &lt; last()]" 
		   mode="equal-one-in-sum">
       <xsl:variable name="output">
	 <sum>
	   <acc>
	     <xsl:copy-of select="(child::*[not(self::diff-equal[@value=1])])[1]"/>
	   </acc>
	   <xsl:copy-of select="child::* except 
				((child::*[not(self::diff-equal[@value=1])])[1] 
				 |(child::diff-equal[@value=1])[1])"/>
	 </sum>
       </xsl:variable>
       <xsl:message>
	 elim-one-in-sum <xsl:copy-of select="."/>
	 to <xsl:copy-of select="$output"/>
       </xsl:message>
       <xsl:copy-of select="$output"/>
     </xsl:template>

     <xsl:template match="*" mode="equal-one-in-sum">
	 <xsl:copy-of select="."/>
     </xsl:template>

</xsl:stylesheet>
