<xsl:stylesheet 
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:strip-space elements="*" />
    
    <xsl:template match="//difference-constraints">
      <xsl:variable name="start-constraint">
	<difference-constraints change="false">
	  <xsl:copy-of select="@*| child::*"/>
	</difference-constraints>
      </xsl:variable>
      <xsl:apply-templates select="$start-constraint//difference-constraints" mode="simplify"/>
    </xsl:template>

    <xsl:template match="difference-constraints" mode="simplify">
      <xsl:choose>
	<xsl:when test="@change='false'">
	  <xsl:apply-templates select="." mode="elim-equal"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="."/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

<!--
      <xsl:variable name="zero-sum-simplified-equations">
	<xsl:apply-templates select="$presimplified-equations//steady-state" 
			     mode="zero-equal-sum-simplify"/>
      </xsl:variable>

      <xsl:variable name="minus-minus-simplified-equations">
     	<xsl:apply-templates select="$zero-sum-simplified-equations//steady-state"
			    mode="minus-minus-simplify"/>
      </xsl:variable>
      <xsl:apply-templates select="$minus-minus-simplified-equations//steady-state" 
			   mode="elim-equal"/>
      
-->
 

     <xsl:template match="steady-state" mode="simplify">
       <steady-state debug="weakly simplified">
	 <xsl:apply-templates select="equal|similar" mode="simplify"/>
       </steady-state>
     </xsl:template>
     <xsl:template match="equal" mode="simplify">
       <equal>
	 <xsl:apply-templates select="child::*" mode="simplify"/>
       </equal>
     </xsl:template>
     <xsl:template match="similar" mode="simplify">
       <similar>
	 <xsl:apply-templates select="child::*" mode="simplify"/>
       </similar>
     </xsl:template>
     <xsl:template match="speed|conc|constant|input|output" mode="simplify">
       <xsl:copy-of select="."/>
     </xsl:template>
     <xsl:template match="mult" mode="simplify">
       <xsl:if test="count(child::*) = 1">
	 <xsl:apply-templates select="child::*" mode="simplify"/>
       </xsl:if>
      <xsl:if test="count(child::*) = 0">
	 <constant value="1"/>
      </xsl:if>
      <xsl:if test="count(child::*) > 1">
	<mult>
	  <xsl:apply-templates select="child::*" mode="simplify"/>
	</mult>
      </xsl:if>
     </xsl:template>

     <xsl:template match="sum" mode="simplify">
       <xsl:if test="count(child::*) = 1">
	 <xsl:apply-templates select="child::*" mode="simplify"/>
       </xsl:if>
      <xsl:if test="count(child::*) = 0">
	 <constant value="0"/>
      </xsl:if>
      <xsl:if test="count(child::*) > 1">
	<sum>
	  <xsl:apply-templates select="child::*" mode="simplify"/>
	</sum>
      </xsl:if>
     </xsl:template>
     <xsl:template match="inh" mode="simplify">
       <xsl:if test="count(child::*) >= 1">
	 <inh>
	   <xsl:apply-templates select="child::*" mode="simplify"/>
	 </inh>
       </xsl:if>
      <xsl:if test="count(child::*) = 0"/>
     </xsl:template>
     <xsl:template match="minus" mode="simplify">
       <minus>
	 <xsl:apply-templates select="child::*" mode="simplify"/>
       </minus>
     </xsl:template>

     <xsl:template match="steady-state" mode="zero-equal-sum-simplify">
       <steady-state debug="zero-equal-sum-simplied">
	 <xsl:apply-templates select="equal[sum and constant[@value='0']]" 
			      mode="zero-equal-sum-simplify"/>
	 <xsl:copy-of select="similar|equal[not(sum) or not(constant[@value='0'])]"/> 
       </steady-state>
     </xsl:template>
     
     <xsl:template match="equal[sum and constant[@value='0']]" 
		  mode="zero-equal-sum-simplify">
        <xsl:variable name="rest-sum">
	 <sum>
	   <xsl:copy-of select="sum/child::*[position()>1]"/>
	 </sum>
       </xsl:variable>
       <xsl:variable name="first">
	 <xsl:copy-of select="sum/child::*[1]"/>
       </xsl:variable>
       <equal>
	 <xsl:choose>
	   <xsl:when test="$first//minus">
	     <xsl:copy-of select="$first//minus/*"/>
	   </xsl:when>
	   <xsl:otherwise>
	     <minus>
	       <xsl:copy-of select="$first"/>
	     </minus>
	   </xsl:otherwise>
	 </xsl:choose>
	 <xsl:apply-templates select="$rest-sum//sum" mode="simplify"/> 
       </equal>
     </xsl:template>


     <xsl:template match="steady-state" mode="minus-minus-simplify">
       <steady-state debug="minus-minus-simplied">
	 <xsl:apply-templates select="equal[child::*[1]/self::minus]
				      [child::*[2]/self::minus]" 
			      mode="minus-minus-simplify"/>
	 <xsl:copy-of select="similar|equal[not(child::*[1]/self::minus) or
			      not(child::*[2]/self::minus)]"/>
       </steady-state>
     </xsl:template>
     
     <xsl:template match="equal[child::*[1]/self::minus]
				      [child::*[2]/self::minus]"
		  mode="minus-minus-simplify">
       <equal>
	 <xsl:copy-of select="child::minus/*"/>
       </equal>
     </xsl:template>

     <xsl:template match="difference-constraints" mode="elim-equal">
       <debug-enter-elim-equal>
	 <xsl:copy-of select="."/>
       </debug-enter-elim-equal>
       <xsl:choose>
	 <xsl:when test="equal[count(speed|conc)>0 and 
			 count(conc|speed|input|output)=2]">
	   <difference-constraint change="yes">
	     <xsl:for-each 
		 select="(equal[count(speed|conc)>0 
			 and count(speed|conc|input|output)=2])[1]">
	       <xsl:variable name="rest-equations">
		 <steady-state debug="elim-equalities">
		   <xsl:copy-of select="preceding-sibling::*|following-sibling::*"/>
		 </steady-state>
	       </xsl:variable>
	       <xsl:variable name="next-equations">
		 <xsl:apply-templates 
		     select="$rest-equations//difference-constraints" 
		     mode="query-replace">
		   <xsl:with-param name="query"   select="(speed|conc)[1]"/>
		   <xsl:with-param name="replace" select="input|output|(speed|conc)[2]"/>
	       </xsl:apply-templates>
	       </xsl:variable>
	       <xsl:apply-templates select="$next-equations//difference-constraints" mode="elim-equal"/>
	     </xsl:for-each>
	   </difference-constraints>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

     <xsl:template match="@*" mode="query-replace">
       <xsl:copy-of select="."/>
     </xsl:template>

     <xsl:template match="*[not(self::speed)]" mode="query-replace">
       <xsl:param name="query"/>
       <xsl:param name="replace"/>
       
       <xsl:variable name="tag">
	 <xsl:value-of select="name(.)"/>
       </xsl:variable>
       
      <xsl:element name="{$tag}">
	<xsl:apply-templates select="child::*|@*" mode="query-replace">
	  <xsl:with-param name="query"   select="$query"/>
	  <xsl:with-param name="replace" select="$replace"/>
	</xsl:apply-templates>
      </xsl:element>

     </xsl:template>

     <xsl:template match="speed" mode="query-replace">
       <xsl:param name="query"/>
       <xsl:param name="replace"/>
       <xsl:choose>
	 <xsl:when test="@reaction=$query/@reaction">
	   <!--
	   <HERE>
	     <xsl:copy-of select="$query"/>
	   </HERE>
	   -->
	   <xsl:copy-of select="$replace"/>
	 </xsl:when>
	 <xsl:otherwise>
	   <xsl:copy-of select="."/>
	 </xsl:otherwise>
       </xsl:choose>
     </xsl:template>

</xsl:stylesheet>
