<xsl:stylesheet 
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0"
    exclude-result-prefixes="xs">

    <xsl:output   
	method="text" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:strip-space elements="*" />
      
    <xsl:template match="/">

      
      <xsl:variable name="full-id">
	<xsl:value-of select="network/@id"/>
	<xsl:if test="network/@kind">
	  <text>-</text>
	  <xsl:value-of select="network/@kind"/>
	</xsl:if>
      </xsl:variable>

      <text xml:space="preserve">\renewcommand\TIKZID[1]{</text>
      <xsl:value-of select="$full-id"/>
      <text xml:space="preserve">}
\renewcommand\TIKZREFID[1]{</text>
      <xsl:value-of select="$full-id"/>
      <text>}

\begin{tabular}{|l||l|l|}
\hline
Role      &amp; Short name &amp; Chemical Species
\\\hline\hline
</text>

      <xsl:apply-templates select="//network"/>

      <text xml:space="preserve">\hline\hline
\end{tabular}
</text>
      
    </xsl:template>
    
    <xsl:template match="//network"> 
      <xsl:variable name="molecules">
	<xsl:apply-templates select="metabolite[1]"/>
	<xsl:apply-templates select="metabolite[position() > 1]"/>
	<xsl:apply-templates select="protein[1]"/>
	<xsl:apply-templates select="protein[position() > 1]"/>
	<xsl:apply-templates select="actor[1]"/>
	<xsl:apply-templates select="actor[position() > 1]"/>

      </xsl:variable>

      <xsl:value-of select="$molecules"/>
    </xsl:template>
    
   <xsl:template match="metabolite[1]">

       <text xml:space="preserve"> Metabolites &amp; \Mtable{<xsl:value-of select="normalize-space(@id)"/>} &amp; <xsl:value-of select="@comment"/>
\\
</text>


   <xsl:for-each select="comment">
     <text xml:space="preserve"> &amp; &amp; <xsl:value-of select="@latex"/>
\\
</text>
   </xsl:for-each>
   </xsl:template>

   <xsl:template match="metabolite[position() > 1]">


       <text xml:space="preserve"> &amp; \Mtable{<xsl:value-of select="normalize-space(@id)"/>} &amp; <xsl:value-of select="@comment"/>
\\
</text>


   <xsl:for-each select="comment">
     <text xml:space="preserve"> &amp; &amp; <xsl:value-of select="@latex"/>
\\
</text>
   </xsl:for-each>
   </xsl:template>


   <xsl:template match="protein[1]">

       <text xml:space="preserve"> Proteines &amp; \Mtable{<xsl:value-of select="normalize-space(@id)"/>} &amp; <xsl:value-of select="@comment"/>
\\
</text>

   <xsl:for-each select="comment">
     <text xml:space="preserve"> &amp; &amp; <xsl:value-of select="@latex"/>
\\
</text>
   </xsl:for-each>
   </xsl:template>

   <xsl:template match="protein[position() > 1]">

       <text xml:space="preserve"> &amp; \Mtable{<xsl:value-of select="normalize-space(@id)"/>} &amp; <xsl:value-of select="@comment"/>
\\
</text>

   <xsl:for-each select="comment">
     <text xml:space="preserve"> &amp; &amp; <xsl:value-of select="@latex"/>
\\
</text>
   </xsl:for-each>
   </xsl:template>

 <xsl:template match="actor[1]">

       <text xml:space="preserve"> Actors &amp; \Mtable{<xsl:value-of select="normalize-space(@id)"/>} &amp; <xsl:value-of select="@comment"/>
\\
</text>

   <xsl:for-each select="comment">
     <text xml:space="preserve"> &amp; &amp; <xsl:value-of select="@latex"/>
\\
</text>
   </xsl:for-each>
   </xsl:template>

   <xsl:template match="actor[position() > 1]">

       <text xml:space="preserve"> &amp; \Mtable{<xsl:value-of select="normalize-space(@id)"/>} &amp; <xsl:value-of select="@comment"/>
\\
</text>


   <xsl:for-each select="comment">
     <text xml:space="preserve"> &amp; &amp; <xsl:value-of select="@latex"/>
\\
</text>
   </xsl:for-each>
   </xsl:template>

</xsl:stylesheet>
