<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0">

    <xsl:output   
	method="xml" 
	version="1.0"
	encoding="utf-8"
	indent="yes"/>
    
    <xsl:param name="MACROS" required="yes"/>

    <xsl:template match="/*">
      <xsl:variable name="next">
	<xsl:apply-templates select="." mode="substitute-step"/>
      </xsl:variable>
      <xsl:choose>
	<xsl:when test="count(//macro)!=0">
	  <!-- debugging 
	       <xsl:copy-of select="$next"/> 
	       <xsl:text> 
	       
	       NEXT STEP
	       
	       </xsl:text>
	  -->
	  <xsl:apply-templates select="$next"/>
	</xsl:when>
	<xsl:otherwise>
	  <!--
	  <xsl:copy-of select="$next"/> 
	  <xsl:text> 
	    
	    MAKE LINKS
	    
	  </xsl:text>
	  -->
	  <xsl:apply-templates select="$next" mode="make-links"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <!-- step of macro substitution -->

    <xsl:template match="macro[@type]" mode="substitute-step">
      <xsl:param name="name" select="@name"/>
      <xsl:param name="type" select="@type"/>
      <xsl:copy-of select="document($MACROS)//def-macro[@name=$name][@type=$type]/(node()|*)"/> 
    </xsl:template>
    
    <xsl:template match="macro[not(@type)]" mode="substitute-step">
      <xsl:param name="name" select="@name"/>
      <xsl:copy-of select="document($MACROS)//def-macro[@name=$name][not(@type)]/(node()|*)"/> 
    </xsl:template>
    
    <xsl:template match="Telefon[@Land]" mode="substitute-step">

      <xsl:param name="Land" select="@Land"/>
      <xsl:param name="LandPrefix" select="document($MACROS)//def-macro[@name=$Land]/text()"/>

      <xsl:choose> 
	<xsl:when test="normalize-space($LandPrefix)=''">
	    <Bemerkung> 	  
	      <font color="red">
		Warning empty LandPrefix for <xsl:value-of select="$Land"/>:<xsl:copy-of select="$LandPrefix"/>:
		<!-- debug geht aber nicht
		<xsl:apply-templates select="document($MACROS)" mode="copy"/>
		-->
	      </font>
	    </Bemerkung>
	</xsl:when>
      </xsl:choose> 
      
      <Telefon>
	<xsl:attribute name="Nummer" select="concat($LandPrefix,@Nummer)"/>
	<xsl:copy-of select="@*[name()!='Nummer'][name()!='Land']"/>
	<xsl:apply-templates select="node()|*" mode="substitute-step"/> 
      </Telefon>
    </xsl:template>

    <xsl:template match="Fax[@Land]" mode="substitute-step">
      <xsl:param name="Land" select="@Land"/>
      <xsl:param name="LandPrefix" select="document($MACROS)//def-macro[@name=$Land]/text()"/>
      <xsl:choose> 
	<xsl:when test="normalize-space($LandPrefix)=''">
	  <Bemerkung> 	  
	    <font color="red">
	      Warning empty LandPrefix for <xsl:value-of select="$Land"/>:<xsl:copy-of select="$LandPrefix"/>:fax
	    </font>
	  </Bemerkung>
	</xsl:when>
      </xsl:choose> 
      <Fax>
	<xsl:attribute name="Nummer" select="concat($LandPrefix,@Nummer)"/>
	<xsl:copy-of select="@*[name()!='Nummer'][name()!='Land']"/>
      </Fax>
    </xsl:template>

    <xsl:template match="Email[@Name]" mode="substitute-step">
      <xsl:param name="Name" select="@Name"/>
      <xsl:param name="Email" select="document($MACROS)//def-macro[@name=$Name][@type='email']/(node()|*)"/>
      <Email>
	<xsl:copy-of select="@*[name()!='Name']"/>
	<xsl:copy-of select="$Email"/> 
      </Email>
    </xsl:template>

    <xsl:template match="Web[@Name]" mode="substitute-step">
      <xsl:param name="Name" select="@Name"/>
      <xsl:param name="Web" select="document($MACROS)//def-macro[@name=$Name][not(@type)]/(node()|*)"/>
      <Web>
	<xsl:copy-of select="@*[name()!='Name']"/>
	<xsl:copy-of select="$Web"/> 
      </Web>
    </xsl:template>

    <xsl:template match="*" mode="substitute-step">
      <xsl:copy>
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="node()|*" mode="substitute-step"/>
      </xsl:copy>
    </xsl:template>

    <xsl:template match="text()|@*" mode="substitute-step">
      <xsl:value-of select="."/>
    </xsl:template>

 <!--make links -->

    <xsl:template match="link[@name]" mode="make-links">
      <xsl:param name="name" select="@name"/>
      <xsl:param name="href">
	<xsl:value-of select="document($MACROS)//def-macro[@name=$name][not(@type)]/(node()|*)"/>
      </xsl:param>
      <a>
	<xsl:attribute name="href">
	  <xsl:value-of select="normalize-space(replace($href,'&#xA;',''))"/>
	</xsl:attribute>
	<xsl:value-of select="$name"/>
      </a>
    </xsl:template>

    <xsl:template match="*" mode="make-links">
      <xsl:copy>
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="node()|*" mode="make-links"/>
      </xsl:copy>
    </xsl:template>

    <xsl:template match="text()|@*" mode="make-links">
      <xsl:value-of select="."/>
    </xsl:template>

    <!-- output macros for debugging
    <xsl:template match="*" mode="copy">
      <xsl:copy>
	<xsl:copy-of select="@*"/>
	<xsl:apply-templates select="node()|*" mode="make-links"/>
      </xsl:copy>
    </xsl:template>
    --> 
</xsl:stylesheet>
