<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet 
    xmlns="http://www.w3.org/1999/xhtml"   
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    version="2.0">
<xsl:output method="html" version="5.0" encoding="UTF-8" indent="yes" />

  <xsl:param name="VERSION"     required="yes"/>
  <xsl:param name="DATE"        required="yes"/>

  <xsl:template match="document">
    <html lang="en">
      <head>
	<link rel="stylesheet" href="styles.css"/>
	<title>
	  <xsl:value-of select="title"/>
	</title>
      </head>
      
      <body>
	<header>
	  <h1>
	    <xsl:value-of select="title"/>
  	  </h1>
	   <h2>
	     <xsl:apply-templates select="author"/>
	   </h2>
	   <h3>
	      <xsl:text>Version </xsl:text>
	      <xsl:copy-of select="$VERSION"/>
	      <xsl:text> from </xsl:text>
	      <xsl:copy-of select="$DATE"/>
	      <xsl:text>, </xsl:text>
	      <xsl:copy-of select="copyright"/>
	   </h3>

	</header>
		   

	<xsl:apply-templates select="section"/>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="author">
    <xsl:apply-templates select="*|text()"/>
  </xsl:template>
  
  <xsl:template match="section">
    <section>
      <header>
	<h2>
	  <xsl:value-of select="@name"/>
	</h2>
      </header>
      <xsl:apply-templates select="*|text()"/>
    </section>
  </xsl:template>
  <xsl:template match="subsection">
    <section>
      <header>
	<h3>
	  <xsl:value-of select="@name"/>
	</h3>
      </header>
      <xsl:apply-templates select="*|text()"/>
    </section>
  </xsl:template>

  <xsl:template match="list">
    <xsl:if test="@name"> 
      <h3> <xsl:value-of select="@name"/></h3> 
    </xsl:if>
    <ul>
    <xsl:apply-templates select="*"/> </ul>
  </xsl:template>
  
  <xsl:template match="item">
    <li> <xsl:apply-templates select="node()"/> </li> 
  </xsl:template>

  <xsl:template match="reaction">
    r<sub><xsl:value-of select="@id"/></sub>
  </xsl:template>
  <xsl:template match="spec">
    <code><xsl:value-of select="@id"/></code>
  </xsl:template>
  <xsl:template match="BS">
    <code>BS<sub><xsl:value-of select="@spec"/></sub></code>
  </xsl:template>
  <xsl:template match="Prom">
    <code>P<sub><xsl:value-of select="@spec"/></sub></code>
  </xsl:template>
  
  <xsl:template match="article">
    <article>
      <header>
	<h4> <xsl:value-of select="@name"/> </h4>
      </header>
      <xsl:apply-templates select="@*[not(name()='name')]|*|text()"/>
    </article>
  </xsl:template>
  
  <xsl:template match="file">
    <xsl:variable name="filename">
      <xsl:value-of select="@name"/>
    </xsl:variable>
    <xsl:variable name="file">
      <xsl:copy-of select="document($filename)"/>
    </xsl:variable>
    <xsl:apply-templates select="$file//document/*"/>
  </xsl:template>

  <xsl:template match="xml-code">
    <xsl:variable name="filename">
      <xsl:value-of select="concat('Created/',@name,'/include.xml')"/>
    </xsl:variable>
    <xsl:variable name="file">
       <xsl:copy-of select="document($filename)"/>
    </xsl:variable>
    <xsl:apply-templates select="$file//xml"/>
  </xsl:template>
  
   <xsl:template match="declare">
     <p class="declare">
       <code class="declare"> <xsl:value-of select="@name"/> </code>:
       <xsl:value-of select="@by"/>
       <xsl:apply-templates select="*|text()"/>
     </p>
   </xsl:template>
   <xsl:template match="a">
     <a>
       <xsl:apply-templates select="@*|*|text()"/>
     </a>
   </xsl:template>
    <xsl:template match="p">
      <p>
	<xsl:if test="@name">
	  <h4> <xsl:value-of select="@name"/> </h4>
	</xsl:if>
	<xsl:apply-templates select="*|text()"/>
      </p>
   </xsl:template>
   <xsl:template match="div">
     <div>
       <xsl:apply-templates select="*|text()"/>
     </div>
   </xsl:template>
   <xsl:template match="dtd">
     <div class="dtd">
       <pre>
	 <code>
	   <xsl:apply-templates select="*|comment()|text()"/>
	 </code>
       </pre>
     </div>
   </xsl:template>
   <xsl:template match="comment()">
     &lt;!-- <xsl:value-of select="."/> --&gt;
   </xsl:template>
   <xsl:template match="xml">
     <div class="xml">
       <pre>
	   <xsl:value-of select="."/>
       </pre>		 
     </div>
   </xsl:template>   <xsl:template match="make">
     <div class="make">
       <pre>
	   <xsl:value-of select="."/>
       </pre>
     </div>
   </xsl:template>
   <xsl:template match="pre">
     <pre>
       <xsl:apply-templates select="@*|*"/>
      </pre>
   </xsl:template>
   <xsl:template match="code">
     <code>
       <xsl:value-of select="."/>
      </code>
   </xsl:template>
   <xsl:template match="center">
     <center> <xsl:apply-templates select="*|@*|text()"/> </center>
   </xsl:template>

   <xsl:template match="h2">
     <h2>
	 <xsl:value-of select="@name"/>
     </h2>
     <xsl:apply-templates select="*|text()"/> 
   </xsl:template>
  <xsl:template match="address">
    <xsl:apply-templates select="*"/>
  </xsl:template>
   <xsl:template match="h4">
     <h4>
       <xsl:value-of select="@name"/>
     </h4>
       <xsl:apply-templates select="*|text()"/>
   </xsl:template>
  <xsl:template match="address">
    <xsl:apply-templates select="*"/>
  </xsl:template>

  <xsl:template match="*">
    <xsl:variable name="tag">
      <xsl:value-of select="name(.)"/>
    </xsl:variable>

    <xsl:element name="{$tag}">
        <xsl:apply-templates select="@*|*|text()"/>
    </xsl:element>
  </xsl:template>

  
  <xsl:template match="text()">
    <xsl:value-of select="."/>
  </xsl:template>  
  <xsl:template match="@*">
    <xsl:copy-of select="."/>
  </xsl:template>
  
</xsl:stylesheet>
