<?xml version="1.0" encoding="utf-8"?>
<iso:schema
  xmlns="http://purl.oclc.org/dsdl/schematron" 
  xmlns:iso="http://purl.oclc.org/dsdl/schematron"
  xmlns:dp="http://www.dpawson.co.uk/ns#"
  xmlns:xs="http://www.w3.org/XMLSchema"
  queryBinding='xslt2'
  schemaVersion='ISO19757-3'>

  <iso:title>Validate reaction networks</iso:title>
  <iso:ns prefix='dp' uri='http://www.dpawson.co.uk/ns#'/> 

  <iso:pattern>
    <iso:rule context="network/child::*[self::reaction or self::context]">

      <iso:assert flag="error" 
		  test="not(@id=preceding-sibling::*[self::reaction or self::context]/@id)">
	Identifiers of reactions, inflows, and outflows must be unique. This is violated in &lt;<iso:value-of select="name()"/> id="<iso:value-of select="@id"/>" ... &gt;  Some other reaction, inflow or outflow has the same id.
      </iso:assert>  
    </iso:rule>
  
    <iso:rule  context="network/child::*[self::protein or self::metabolite or self::actor]">
      <iso:assert flag="error"
		  test="not(@id=preceding-sibling::*[self::protein
			or self::metabolite or self::actor]/@id)">
	Identifier of species must be unique. This is violated in: &lt;<iso:value-of select="name()"/> id="<iso:value-of select="@id"/>" ... &gt; Some other metabolites, protein, or actor has the same id.
      </iso:assert>  
    </iso:rule>


        <iso:rule  context="network/child::*[self::protein or self::metabolite or self::actor]/copy">
      <iso:assert flag="error"
		  test="not(@id=preceding-sibling::copy/@id)">
	Identifier of copies must be unique for all species. This is violated in: &lt;<iso:value-of select="parent::*/name()"/> id="<iso:value-of select="parent::*/@id"/>" ... &gt;&lt; copy id=<iso:value-of select="@id"/>" ... &gt; Some other copy has the same id.
      </iso:assert>  
    </iso:rule>


    
    <iso:rule  context="network/child::edgecluster">
      <iso:assert flag="error"
		  test="not(@id=preceding-sibling::edgecluster/@id)">
	Identifier of edgeclusters must be unique. This is violated in: &lt;<iso:value-of select="name()"/> id="<iso:value-of select="@id"/>" ... &gt; Some other edgeclsuter the same id.
      </iso:assert>  
    </iso:rule>


    <iso:rule context="network/child::reaction/child::*[@spec]">
      <iso:assert flag="error"
		  test="@spec=//*[self::protein or self::actor or self::metabolite]/@id">
	Identifier of species used by reaction is not introduced:
	&lt;<iso:value-of select="name()"/> spec="<iso:value-of select="@spec"/>" ... &gt;
      </iso:assert>  
    </iso:rule>

  </iso:pattern>
  
  <iso:pattern>
    <iso:rule context="//*[@id]">
      <iso:assert flag="error"
		  test="matches(@id, '^[\+|\c|\d]+$')">
	Identifiers must be nonempty strings with letter that are alpha-numeric or "-" or "_" or "+". This is violated in:
	&lt;<iso:value-of select="name()"/> id="<iso:value-of select="@id"/>" ... &gt;
      </iso:assert>  
    </iso:rule>
  </iso:pattern>
  <iso:pattern>
    <iso:rule context="network[@kind]">
      <iso:assert flag="error"
		  test="matches(@kind, '^[\+|\c|\d]+$')">
	Kinds of networks must a nonempty strings with letter that are alpha-numeric or "-" or "_" or "+". This is violated in:
	&lt;<iso:value-of select="name()"/> kind="<iso:value-of select="@kind"/>" ... &gt;
      </iso:assert>  
    </iso:rule>
  </iso:pattern>

  <iso:pattern>
    <iso:rule context="//*[@id]">
      <iso:assert flag="error"
		  test="not(starts-with(@id,'r_'))">
	No identifiers may start with 'r_' (reserved for implementation of reactions). This is violated in:
	&lt;<iso:value-of select="name()"/> id="<iso:value-of select="@id"/>" ... &gt;
      </iso:assert>  
    </iso:rule>
  </iso:pattern>  <iso:pattern>
    <iso:rule context="//*[@id]">
      <iso:assert flag="error"
		  test="not(starts-with(@id,'c_'))">
	No identifier may start with 'c_' (reserved for implementation of contexts). This is violated in:
	&lt;<iso:value-of select="name()"/> id="<iso:value-of select="@id"/>" ... &gt;
      </iso:assert>  
    </iso:rule>
  </iso:pattern>  <iso:pattern>
    <iso:rule context="//*[@id]">
      <iso:assert flag="error"
		  test="not(
			starts-with(@id,'copy--') or 
			starts-with(@id,'splitpoint--') or 
			starts-with(@id,'fake--') or 
			starts-with(@id,'edgecluster--')) 
			">
	No identifier may start with 'copy--' or 'splitpoint--' or 'fake--' or 'edgecluster--'. These prefixes are reserved for the implementation of species with copies and edgeclusters. This is violated in:
	&lt;<iso:value-of select="name()"/> id="<iso:value-of select="@id"/>" ... &gt;
      </iso:assert>  
    </iso:rule>
  </iso:pattern>

  <iso:pattern>
    <iso:rule context="//kinetics[@id]">
      <iso:assert flag="error" test="@id='exp' or @id='ma' or @id='deac'">
	Identifiers of kinetics must be either of: expression 'exp', mass action 'ma', or deactivation 'deac'. This is violated in:
	&lt;<iso:value-of select="name()"/> id="<iso:value-of select="@id"/>" ... &gt;
      </iso:assert>  
    </iso:rule>
  </iso:pattern>
    
  <iso:pattern>
    <iso:rule context="//kinetics">
      <iso:assert flag="error" test="not(@expr and @id)">
	The kinetics of an reaction may either have an identifier (for expression id="exp", mass action id="ma", or deactivation id="deac") or an expressions (expr="..." with some value in latex format). This is violated in:
	&lt;<iso:value-of select="parent::*/name()"/> id="<iso:value-of select="parent::*/@id"/>" ... &gt;
      </iso:assert>  
    </iso:rule>
    
  </iso:pattern>

  <iso:pattern>
    <iso:rule context="//kinetics[@mode]">
      <iso:assert flag="error" test="@mode='equal'">
The only possible value of the mode attribute is mode="equal". This is violated in:
&lt;<iso:value-of select="parent::*/name()"/> id="<iso:value-of select="parent::*/@id"/>" ... &gt;
Why? The mode attribute defines the intended interpretation mode for the kinetic information. If the mode attribute is absent, the kinetic information is correct only up to similarity (see research papers on abstract interpretation for predicting network changes: https://hal.inria.fr/hal-01239198). If mode="equal" then kinetic information is exact, so equality is used in the ODEs and steady state equations. 
      </iso:assert>  
    </iso:rule>
    
  </iso:pattern>

  <iso:pattern>
    <iso:rule context="source | inhibitor | activator | accelerator">
      <iso:assert flag="error" test="not(@edgecluster and @spec)">
Not both attributes 'spec' or 'edgecluster' may be present. This is violated in:
<iso:copy-of select="."/> 
Why? A reference to a species is either given by attribute species or by attribute edgecluster, but not by both of them.
      </iso:assert>  
      <iso:assert flag="error" test="@edgecluster or @spec">
At least one of the attributes 'spec' or 'edgecluster" must be present. This is violated in:
&lt;<iso:value-of select="parent::*/name()"/> id="<iso:value-of select="parent::*/@id"/>" ... &gt;
Why? A reference to a species requires either a attribute species or attribute edgecluster.
      </iso:assert>  
      <iso:assert flag="error" test="not(@copy) or @spec">
If there is a 'copy' then there is a 'spec'. This is violated in:
&lt;<iso:value-of select="name()"/>
spec="<iso:value-of select="@spec"/>"
copy="<iso:value-of select="@copy"/>" 
edgecluster="<iso:value-of select="@edgecluster"/>" ... &gt;
Why? A reference to a species requires either a attribute species or attribute edgecluster.
      </iso:assert>  
    </iso:rule>
    
  </iso:pattern>
  
<iso:pattern>
  <iso:rule context="source[@copy and @spec] | inhibitor[@copy and @spec] | activator[@copy and @spec] | accelerator[@copy and @spec]">
      <iso:assert flag="error" test="some $species in //*[self::metabolite or self::protein or self::actor] satisfies (@spec=$species/@id and @copy=$species/copy/@id)">
If there are a 'copy' and a 'spec', and there is a species $species with $species/@id=@spec and $species/copy/@id=@copy. This is violated in:
&lt;<iso:value-of select="name()"/>
spec="<iso:value-of select="@spec"/>"
copy="<iso:value-of select="@copy"/>" 
edgecluster="<iso:value-of select="@edgecluster"/>" ... &gt;
Why? A reference to a species requires either a attribute species or attribute edgecluster.
      </iso:assert>  
    </iso:rule>
</iso:pattern>

<iso:pattern>
  <iso:rule context="*[@x] ">
      <iso:assert flag="error" test="if (@x castable as xs:float) then true() else false()">
The x-axis must be a float. This is violated in:
&lt;<iso:value-of select="name()"/>
id="<iso:value-of select="@id"/>" x="<iso:value-of select="@x"/>" y="<iso:value-of select="@y"/>"... &gt;
      </iso:assert>  
    </iso:rule>
</iso:pattern>

<iso:pattern>
  <iso:rule context="*[@y] ">
      <iso:assert flag="error" test="if (@y castable as xs:float) then true() else false()">
The y-axis must be a float. This is violated in:
&lt;<iso:value-of select="name()"/>
id="<iso:value-of select="@id"/>" x="<iso:value-of select="@x"/>" y="<iso:value-of select="@y"/>"  ... &gt;
      </iso:assert>  
    </iso:rule>
</iso:pattern>
  
</iso:schema>